@@ -1,9 +1,9 @@
 /*
  * ccs_filesystem_test.c
  *
- * Copyright (C) 2005-2010  NTT DATA CORPORATION
+ * Copyright (C) 2005-2011  NTT DATA CORPORATION
  *
- * Version: 1.8.0   2010/11/11
+ * Version: 1.8.0+   2011/02/14
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License v2 as published by the
@@ -280,13 +280,15 @@ int main(int argc, char *argv[])
 		{
 			struct stat sbuf;
 			FILE *fp = NULL;
+			int ret_ignored;
 			snprintf(buf, sizeof(buf) - 1, "zcat - > %s",
 				 dev_ram_path);
 			if (lstat(dev_ram_path, &sbuf) == 0 &&
 			    S_ISBLK(sbuf.st_mode) && MAJOR(sbuf.st_rdev) == 1)
 				fp = popen(buf, "w");
 			if (fp) {
-				fwrite(compressed_ext2_image_sample, 1,
+				ret_ignored =
+					fwrite(compressed_ext2_image_sample, 1,
 				       sizeof(compressed_ext2_image_sample),
 				       fp);
 				pclose(fp);
@@ -619,5 +621,9 @@ int main(int argc, char *argv[])
 	rmdir("/tmp/mount/");
 
 	clear_status();
+	if (0) { /* To suppress "defined but not used" warnings. */
+		write_domain_policy("", 0);
+		write_exception_policy("", 0);
+	}
 	return 0;
 }
